# -*- coding: utf-8 -*-
# ---
# jupyter:
#   jupytext:
#     cell_metadata_json: true
#     text_representation:
#       extension: .py
#       format_name: light
#       format_version: '1.5'
#       jupytext_version: 1.7.1
#   kernelspec:
#     display_name: Python 3
#     language: python
#     name: python3
# ---

# # Compute effective hamiltonian for edge states: honeycomb only
# ---

# Imports
from sympy import *
from sympy.physics.matrices import msigma
from sympy.physics.quantum import TensorProduct
from sympy.physics.quantum.dagger import Dagger

init_printing()

# We make the following calculations for a graphene QH/SC junction. We take the continuum model for an infinite graphene nanoribbon along the $y$-axis in the valley-symmetric representation:
# $$
# H = \hbar v \tau_z \otimes \left[-i \rho_0 \sigma_x \partial_x + \left(k \rho_0 + k_0 \rho_z \right) \otimes \sigma_y \right] - \mu \tau_z \otimes \rho_0 \otimes \sigma_0 + \tau_0 \otimes \rho_0 \otimes v \mathbf{A}(x) \cdot \mathbf{\sigma} + \Delta(x) \tau_x \otimes \rho_0 \otimes \sigma_0
# $$
# with
# $$
# \Delta(x) = \Delta \Theta(x)\\
# $$
# and use the Landau gauge $\mathbf{A}(x) = (0, -Bx) \Theta(-x)$.
#
# From now on we also adopt the $\Gamma$-matrices:
# $$
# \Gamma_0 := \tau_z \otimes \rho_0 \otimes \sigma_0,\\
# \Gamma_1 := \tau_z \otimes \rho_0 \otimes \sigma_x,\\
# \Gamma_2 := \tau_z \otimes \rho_0 \otimes \sigma_y,\\
# \Gamma_3 := \tau_x \otimes \rho_0 \otimes \sigma_0,\\
# \Gamma_4 := \tau_0 \otimes \rho_0 \otimes \sigma_y,\\
# \Gamma_5 := \tau_z \otimes \rho_z \otimes \sigma_y.
# $$
#
# We compute the effective Hamiltonian using first-order in perturbation theory (more details in the next section). First, we compute the zero-energy interface states $\{\phi_i\}$ of an unperturbed Hamiltonian $H_0$. Then we obtain the effective Hamiltonian by simply computing:
# $$
# (H_{eff})_{ij} := \langle \phi_i | H_{pert} | \phi_j \rangle.
# $$
#
# ## Explaining the approach
#
# To get the effective Hamiltonian, we must separate an unperturbed term of $H$, $H_0$, and the perturbation, $H_{pert}$. We choose:
# $$
# H_0 = H|_{k = 0}.
# $$
# Then the zero energy states are the solution of:
# $$
# \partial_x \psi = m(x)\psi,\\
# m(x) = \frac{i \Gamma_1^{-1}}{\hbar v} \left[eBx \Theta(x) \Gamma_4 + \mu \Gamma_0 - \Delta(x) \Gamma_3\right].
# $$
# with solution
# $$
# \psi(x) = \psi_{\alpha}(x) e^{\lambda_{\alpha}(x)}
# $$
# where $\psi_{\alpha}(x)$ and $\lambda_{\alpha}(x)$ are, respectively, the eigenvectors and eigenvalues of
# $$
# M(x) = \int_0^x d \xi m(\xi).
# $$
#
# In practice, we do that by wave-matching the solutions of $x<0$ and $x>0$. We then get our set of eigenmodes and obtain finally the effective hamiltonian.

# ## Set basic stuff

# +
# Pauli matrices
sigma0 = eye(2)
sigma1 = msigma(1)
sigma2 = msigma(2)
sigma3 = msigma(3)

# Gamma matrices
Gamma0 = TensorProduct(sigma3, sigma0, sigma0)
Gamma1 = TensorProduct(sigma3, sigma0, sigma1)
Gamma2 = TensorProduct(sigma3, sigma0, sigma2)
Gamma3 = TensorProduct(sigma1, sigma0, sigma0)
Gamma4 = TensorProduct(sigma0, sigma0, sigma2)
Gamma5 = TensorProduct(sigma3, sigma3, sigma2)

# Define symbols
x = symbols(r"x", real=True)
# Physical constants
hbar, e = symbols(r"\hbar e", real=True, nonzero=True, positive=True)
# Parameters
B, Delta, v, mu = symbols(r"B \Delta v_F \mu", real=True, nonzero=True, positive=True)


def M(x, side):
    if side == "left":
        m = 1j * Gamma1 ** -1 / hbar / v * (e * B * x * Gamma4 + mu * Gamma0)
    elif side == "right":
        m = 1j * Gamma1 ** -1 / hbar / v * (-Delta * Gamma3 + mu * Gamma0)
    M = integrate(m, x).simplify()
    return M


# -

# ## Solve eigenproblem

# Solve eigenproblem
eigs_left = M(x, "left").eigenvects(simplify=True)
eigs_right = M(x, "right").eigenvects(simplify=True)

# Solutions for the left side
eigs_left

# Solutions for the right side
eigs_right

# ## Do wave-matching
# We now construct the general solution at each side and do wave-matching. Note that we want regular functions at spatial infinity, so we neglect the diverging solutions.

# +
# Generate empty vectors
sum_vecs_left = Matrix([[0], [0], [0], [0], [0], [0], [0], [0]])
sum_vecs_right = Matrix([[0], [0], [0], [0], [0], [0], [0], [0]])

# Coefficients for left side
a1, a2, a3, a4 = symbols(r"\alpha_1 \alpha_2 \alpha_3 \alpha_4")
a = [a1, a2, a3, a4]
# Coefficients for the right side
b1, b2, b3, b4 = symbols(r"\beta_1 \beta_2 \beta_3 \beta_4")
b = [b1, b2, b3, b4]

# Sum regular solutions in the left
for n in range(4):
    # IMPORTANT! Here we pick the regular solutions only
    vec_left = eigs_left[0][2][n] * exp(eigs_left[0][0])
    sum_vecs_left += a[n] * vec_left

# Sum regular solutions in the right
for n in range(2):
    # IMPORTANT! Here we pick the regular solutions only
    vec_right1 = eigs_right[0][2][n] * exp(eigs_right[0][0])
    vec_right2 = eigs_right[2][2][n] * exp(eigs_right[2][0])
    sum_vecs_right += b[n] * vec_right1 + b[n + 2] * vec_right2

# Take limit at the interface (x = 0)
lim_left = simplify(sum_vecs_left.limit(x, 0, "-"))
lim_right = simplify(sum_vecs_right.limit(x, 0, "+"))
# -

# Check left-side limit
lim_left

# Check right-side limit
lim_right

# Force simplification for right side
lim_right = ((lim_right * (-1j)).simplify() / (-1j)).simplify()
lim_right

# Solve wave-matching
diff_lim = (
    simplify(lim_left - lim_right)
    .refine()
    .subs(abs(E ** 1.0), abs(E))
    .subs(Delta ** 2.0, Delta ** 2)
    .subs(Delta ** 1.0, Delta ** 1)
)
sol = solve(diff_lim)
sol

# There are only two free parameters, $\alpha_{3,4}$, so the space of solutions can be spanned in terms of two linearly independent solutions:

# +
sol_left = sum_vecs_left.subs(sol).simplify()
sol_right = sum_vecs_right.subs(sol).simplify()
# Force simplification right
sol_right = (sol_right * (1j)).simplify() / (1j)

sol1 = (
    (Heaviside(-x) * sol_left + Heaviside(x) * sol_right)
    .subs(a3, 1)
    .subs(a4, 0)
    .simplify()
)
sol2 = (
    (Heaviside(-x) * sol_left + Heaviside(x) * sol_right)
    .subs(a3, 0)
    .subs(a4, 1)
    .simplify()
)
# -

# Check first basis spinor
sol1

# Check that upper and lower halfs are charge conjugated:

(sol1[0] - 1j * sol1[4].subs(B, -B)).simplify()

(sol1[1] - 1j * sol1[5]).simplify()

# Check second basis spinor
sol2

# Seems we can simplify further those solutions, but will try that later.

# ## Obtain the effective Hamiltonian

# We now can calculate the effective Hamiltonian. The code to do the full calculation is in the cell below. However, solving the integral is quite hard, and we just take a look at the shape of the matrix, which does not require solving the integrals.

# +
k, k0 = symbols(r"k k_0")
pert_kin = hbar * v * (k * Gamma2 + k0 * Gamma5)

hk11 = (Dagger(sol1) @ pert_kin @ sol1).simplify()

hk12 = (Dagger(sol2) @ pert_kin @ sol1).simplify()

hk21 = (Dagger(sol1) @ pert_kin @ sol2).simplify()

hk22 = (Dagger(sol2) @ pert_kin @ sol2).simplify()

H_kin = Matrix([[hk11, hk12], [hk21, hk22]])

repls, exprs = cse(H_kin, ignore=[k, k0])
Heff_kin = exprs[0]
Heff_kin.simplify()
Heff_kin
# -


# We see that, despite the complicated form, the Hamiltonian looks like:
# $$
# H_{eff} = \left(
# \begin{array}{cc}
# \tilde{v} (k +  k_0) && 0\\
# 0 && \tilde{v} (k -  k_0)
# \end{array}
# \right),
# $$
# which is what we expected.

# ## Intervalley scattering
#
# We now add another perturbation, which will do the job of intervalley scattering. This second term will have the form:
# $$
# H_{scatt} = \omega \tau_0 \otimes \rho_1 \otimes \sigma_0.
# $$

# +
w = symbols(r"\omega")
pert_scatt = w * TensorProduct(sigma0, sigma1, sigma0)

hs11 = (Dagger(sol1) @ pert_scatt @ sol1).simplify()

hs12 = (Dagger(sol2) @ pert_scatt @ sol1).simplify()

hs21 = (Dagger(sol1) @ pert_scatt @ sol2).simplify()

hs22 = (Dagger(sol2) @ pert_scatt @ sol2).simplify()

H_scatt = Matrix([[hs11, hs12], [hs21, hs22]])

repls, exprs = cse(H_scatt, ignore=[w])
Heff_valley = exprs[0]
Heff_valley.simplify()
Heff_valley
# -

# So we end up with a the following the effective Hamiltonian
# $$
# H_{eff} = \left(
# \begin{array}{cc}
# \tilde{v} (k + k_0) && \tilde{\omega}\\
# \tilde{\omega} && \tilde{v} (k - k_0)
# \end{array}
# \right)
# $$
