# # Graphene Quantum Hall/Superconductor: transport calculations
# ---

# +
import numpy as np

import matplotlib
import matplotlib.pyplot as plt
from matplotlib import rc
import math

matplotlib.rcParams.update(
    {
        "pgf.texsystem": "pdflatex",
        "font.family": "serif",
        "text.usetex": True,
        "pgf.rcfonts": False,
    }
)

from functools import partial
from modules import infinite_system, utils, system, utils, pauli, functions
import numpy as np
import kwant
import tqdm
import adaptive
import xarray as xr
from os import chdir

# +
dimensions = dict(width=200, lB_min=26)

default_params = dict(Delta=0.05, l=50, spinful=False, mu=0.05, mu_SC=0.05, B=1)


def make_inf_syst(interface, dimensions=dimensions):
    # Start device
    device = infinite_system.Device(
        dimensions=dimensions, scaling_factor=10, interface=interface
    )
    # Make system
    device.make_system(SC_square_bool=True)

    def proj(site):
        x = site.pos[0]
        return (1 - 0.5 * (1 + np.tanh(x / 5))) * pauli.tau_0

    wrap_sys = kwant.wraparound.wraparound(device.syst).finalized()
    density = kwant.operator.Density(syst=wrap_sys, onsite=proj, sum=True)
    return wrap_sys, density


# -

def compute(
    wrap_sys, density, params, default_params=default_params, dimensions=dimensions
):

    from functools import partial

    partial_proj_bands = partial(
        utils.proj_bands,
        params={**dimensions, **default_params, **params},
        wrap_sys=wrap_sys,
        density=density,
    )

    def uniform_sampling_1d(xs, ys):
        dx = xs[1] - xs[0]
        return dx

    learner = adaptive.Learner1D(
        partial_proj_bands,
        bounds=(-np.pi, np.pi),
        loss_per_interval=uniform_sampling_1d,
    )

    npoints_goal = lambda l: l.npoints >= 2000
    runner = adaptive.BlockingRunner(learner, goal=npoints_goal)

    #     runner.live_info(update_interval=10)

    data = learner.data
    ks, es_xs = map(np.array, zip(*sorted(data.items())))

    e_ks = es_xs[:, 0, :]
    xpect_ks = es_xs[:, 1, :]

    dataset = xr.Dataset(
        data_vars={
            "energies": (("kpoints", "values"), e_ks),
            "xpect": (("kpoints", "values"), xpect_ks),
        },
        coords={"kpoints": ks, "values": np.linspace(0, len(e_ks[0]), len(e_ks[0]))},
    )

    return dataset


def plot_bands(dataset):
    xr.plot.scatter(
        dataset,
        "kpoints",
        "energies",
        hue="xpect",
        cmap="Greys",
        s=1,
        alpha=0.25,
        add_guide=False,
    )
    plt.xlim(-math.pi, math.pi)
    plt.ylim(-0.2, 0.2)
    plt.ylabel(r"$E(k) - E_F\ [t]$")
    plt.xlabel(r"$k\ \left[\frac{1}{a} \right]$")
    plt.xticks(
        [-np.pi, -np.pi / 2, 0, np.pi / 2, np.pi],
        [r"$-\pi$", r"$-\pi/2$", r"$0$", r"$\pi/2$", r"$\pi$"],
    )
    #     plt.tight_layout()
    plt.savefig("bands.png")
    plt.show()


systems = {
    "armchair": {"params": dict(), "interface": "armchair"},
    "zigzag": {"params": dict(), "interface": "zigzag"},
    "fermi_mismatch": {"params": dict(mu=0.1, mu_SC=0.5), "interface": "zigzag"},
}

chdir("files/bands")
for system in systems:
    chdir(system)
    syst_info = systems[system]
    print("Calculating setup: " + system)
    wrap_sys, density = make_inf_syst(
        interface=syst_info["interface"], dimensions=dimensions
    )
    dataset = compute(wrap_sys=wrap_sys, density=density, params=syst_info["params"])
    plot_bands(dataset)
    dataset.to_netcdf("dataset_" + system + ".nc")
    chdir("..")
