# # Plots

# ### Imports

# +
import numpy as np

import matplotlib
import matplotlib.pyplot as plt
from matplotlib import rc

matplotlib.use("pgf")
matplotlib.rcParams.update(
    {
        "pgf.texsystem": "xelatex",
        "font.family": "serif",
        "text.usetex": True,
        "pgf.rcfonts": False,
    }
)

from functools import partial
from modules import infinite_system, utils, system, utils, pauli, functions
import numpy as np
import kwant
import tqdm

import matplotlib as mpl
import os
import xarray as xr


# -

# ### Disorder


def plot_system(ax):

    from modules import functions
    import math

    ## System dimensions (all in [nm])
    dimensions = dict(
        L_contact=250,  ## Lenght of SC region
        L_device=500,  ## Scattering region lenght
        W_device=606,  ## Width of SC region and right lead
        W_lead=250,  ## Width of upper and lower leads
        lB_min=30,  ## Minimum value of magnetic length (for 1T)
        theta=0,  # np.pi / 6
    )

    # List with values of magnetic field
    Bs = np.linspace(1, 6, num=100)

    # List with values of chemical potential
    mus = np.linspace(1e-3, 0.2, num=100)

    ## Scaling factor
    scaling_factor = 50

    # Edge along x-axis
    interface = "armchair"

    # Initialize device
    device = system.Device(
        dimensions=dimensions,
        scaling_factor=scaling_factor,
        interface=interface,
        spinful=True,
    )

    ## Default parameters
    default_params = dict(
        Delta=1.3e-3 / device.t,  ## Superconducting gap [t]
        Delta_lead=1.3e-3 / device.t,  ## Superconducting gap at the lead [t]
        mu_SC=None,  ## Chemical potential at the superconducting region ('None' means that it is the same of normal region)
        delta_interface=None,  ## Zero for no interface disorder
        delta_boundary=None,  ## Zero for no boundary disorder
        l=50,  ## Smoothing lenght
        spinful=True,
    )

    # Create device
    device.make_system()

    def color(site):
        def disorder_interface(site):
            """
            Set region for disordered interface.

            Parameters:
            -----------

            site : Kwant site
            """
            x, y = site.pos
            return -dimensions["lB_min"] <= x <= dimensions["L_contact"]

        def disorder_boundary(site):
            """
            Set region for boundary disorder.

            Parameters:
            -----------

            site : Kwant site
            """
            x, y = site.pos
            upper = (
                dimensions["W_device"]
                - dimensions["lB_min"]
                + math.tan(dimensions["theta"]) * x
            )
            lower = dimensions["lB_min"] + math.tan(dimensions["theta"]) * x

            x_condition = (
                -dimensions["L_device"] + dimensions["W_lead"] - dimensions["lB_min"]
                < x
                < -dimensions["lB_min"]
            )
            y_condition = not (lower < y < upper)

            return x_condition and y_condition

        if disorder_interface(site):
            delta_mu_interface = kwant.digest.uniform(repr(site.pos)) + 1e-8
        else:
            delta_mu_interface = 1e-8

        if disorder_boundary(site):
            delta_mu_boundary = -kwant.digest.uniform(repr(site.pos)) - 1e-8
        else:
            delta_mu_boundary = 1e-8

        return delta_mu_interface + delta_mu_boundary

    kwant.plot(
        device.syst,
        site_lw=0.0,
        site_color=color,
        site_size=color,
        cmap="RdBu_r",
        colorbar=False,
        ax=ax,
        show=False,
    )


# +
# %matplotlib widget
import matplotlib.pylab as pl
import matplotlib.gridspec as gridspec

# Create 2x2 sub plots
gs1 = gridspec.GridSpec(2, 2)

rc("font", **{"family": "sans-serif", "sans-serif": ["Helvetica"]})
rc("text", usetex=True)
plt.rcParams["lines.linewidth"] = 0.654
plt.rcParams["font.size"] = 18
plt.rcParams["legend.fontsize"] = 18

fig = plt.figure(figsize=(8, 10))
ax1 = fig.add_subplot(gs1[0, :])
ax2 = fig.add_subplot(gs1[1, 0])
ax3 = fig.add_subplot(gs1[1, 1])

## System dimensions (all in [nm])
dimensions = dict(
    L_contact=250,  ## Lenght of SC region
    L_device=500,  ## Scattering region lenght
    W_device=606,  ## Width of SC region and right lead
    W_lead=250,  ## Width of upper and lower leads
    lB_min=30,  ## Minimum value of magnetic length (for 1T)
    theta=0,  # np.pi / 6
)

plot_system(ax=ax1)

ax1.set_ylim(
    -dimensions["W_lead"] / 2, dimensions["W_device"] + dimensions["W_lead"] / 2
)

ax1.axis("off")
ax1.set_aspect("equal")

import matplotlib

patch1 = matplotlib.patches.FancyBboxPatch(
    (-dimensions["L_device"], -dimensions["W_lead"]),
    dimensions["W_lead"],
    dimensions["W_lead"],
    boxstyle="round",
    bbox_transmuter=None,
    mutation_scale=50.0,
    mutation_aspect=None,
    fc="lightgray",
    zorder=100,
)
patch2 = matplotlib.patches.FancyBboxPatch(
    (-dimensions["L_device"], dimensions["W_device"]),
    dimensions["W_lead"],
    dimensions["W_lead"],
    boxstyle="round",
    bbox_transmuter=None,
    mutation_scale=50.0,
    mutation_aspect=None,
    fc="lightgray",
    zorder=100,
)

ax1.add_patch(patch1)
ax1.add_patch(patch2)

dataset_GNS_boundary = xr.open_dataset(
    "./files/transport/spinless/honeycomb_sc/data/GNS_mu_B_boundary_disorder.nc"
)
dataset_GNS_interface = xr.open_dataset(
    "./files/transport/spinless/honeycomb_sc/data/GNS_mu_B_interface_disorder.nc"
)

# Conductance plots
utils.plot_GNS(dataset_GNS_boundary, fig=fig, ax=ax2, show_clb=False, set_fonts=False)

im = utils.plot_GNS(
    dataset_GNS_interface,
    fig=fig,
    ax=ax3,
    show_y_label=False,
    show_clb=False,
    set_fonts=False,
)

ax1.text(-0.1, 0.9, r"$(a)$", transform=ax1.transAxes)
ax2.text(0.0, 1.05, r"$(b)$", c="k", transform=ax2.transAxes)
ax3.text(0.0, 1.05, r"$(c)$", c="k", transform=ax3.transAxes)

# Add colorbars
clb = fig.colorbar(
    im, ax=(ax1, ax2, ax3), orientation="horizontal", shrink=0.75, aspect=30
)
clb.set_label(r"$G_{xx}\ \left[ \frac{e^2}{h} \right]$", labelpad=0)
clb.set_ticks([-2, -1, 0, 1, 2])

ax3.set_yticklabels([])

fig.savefig("./plots/fig3.pdf", bbox_inches="tight")
fig.show()
# -


