# # Plots

# ### Imports

# +
import numpy as np

import matplotlib
import matplotlib.pyplot as plt
from matplotlib import rc

matplotlib.use("pgf")
matplotlib.rcParams.update(
    {
        "pgf.texsystem": "xelatex",
        "font.family": "serif",
        "text.usetex": True,
        "pgf.rcfonts": False,
    }
)

from functools import partial
from modules import infinite_system, utils, system, utils, pauli, functions
import numpy as np
import kwant
import tqdm

import matplotlib as mpl
import os
import xarray as xr

# -

# ### Valley vs. electron-hole

# +
### from modules import functions
import math

## System dimensions (all in [nm])
dimensions = dict(
    L_contact=250,  ## Lenght of SC region
    L_device=500,  ## Scattering region lenght
    W_device=606,  ## Width of SC region and right lead
    W_lead=250,  ## Width of upper and lower leads
    lB_min=30,  ## Minimum value of magnetic length (for 1T)
    theta=0,  # np.pi / 6
)

# List with values of magnetic field
Bs = np.linspace(1, 6, num=100)

# List with values of chemical potential
mus = np.linspace(1e-3, 0.2, num=100)

## Scaling factor
scaling_factor = 10

# Edge along x-axis
interface = "armchair"

# Initialize device
device = system.Device(
    dimensions=dimensions,
    scaling_factor=scaling_factor,
    interface=interface,
    spinful=False,
)

## Default parameters
default_params = dict(
    Delta=1.3e-3 / device.t,  ## Superconducting gap [t]
    Delta_lead=1.3e-3 / device.t,  ## Superconducting gap at the lead [t]
    mu_SC=None,  ## Chemical potential at the superconducting region ('None' means that it is the same of normal region)
    delta_interface=None,  ## Zero for no interface disorder
    delta_boundary=None,  ## Zero for no boundary disorder
    l=50,  ## Smoothing lenght
    spinful=False,
)

# Create device
device.make_system()

# +
# Initialize device
device2 = system.Device(
    dimensions=dimensions, scaling_factor=50, interface=interface, spinful=False
)

# Create device
device2.make_system()

# +
# Two different values of magnetic field
B1 = 1
B2 = 6
# Parameters
params = dict(mu=0.075)

# Compute wavefunction
def wf(params, B):
    params["B"] = B
    wavefunction = kwant.solvers.default.wave_function(
        sys=device.fsyst, energy=0.0, params={**default_params, **params, **dimensions}
    )
    return wavefunction


# Calculate wavefuntion for desired magnetic field
wf1 = wf(params, B1)
wf2 = wf(params, B2)

# #%%px --local
from modules import valley

valley.valley_operator(device)
v1 = device.operator.hamiltonian_submatrix(
    sparse=True, params={**default_params, **dict(B=B1)}
)
v2 = device.operator.hamiltonian_submatrix(
    sparse=True, params={**default_params, **dict(B=B2)}
)


def valley_density(psi, valley_op):
    # Define density operator
    valley_density = kwant.operator.Density(device.fsyst)
    # Calculate valley density
    valley = valley_density(psi, valley_op @ psi) + valley_density(valley_op @ psi, psi)
    return valley


# Valley density for desired chemical potentials
psi1 = wf1(1)[0]
psi2 = wf2(1)[0]
valley1 = valley_density(psi1, v1)
valley2 = valley_density(psi2, v2)

from modules.pauli import tau_3

# Define electron-hole local operator
eh_density = kwant.operator.Density(device.fsyst, -tau_3)
# Electron-hole density for desired chemical potentials
eh1 = eh_density(psi1)
eh2 = eh_density(psi2)

# +
# %matplotlib widget

# from modules import plot
import matplotlib.pylab as pl
import matplotlib.gridspec as gridspec

# Create 1x2 sub plots
gs = gridspec.GridSpec(1, 2, wspace=0.1)

rc("font", **{"family": "sans-serif", "sans-serif": ["Helvetica"]})
rc("text", usetex=True)
plt.rcParams["lines.linewidth"] = 0.654
plt.rcParams["font.size"] = 20
plt.rcParams["legend.fontsize"] = 20

fig = plt.figure(figsize=(10, 4))
ax1 = fig.add_subplot(gs[0, 0])
ax2 = fig.add_subplot(gs[0, 1])

# Plot valley number

valley_density = valley1.real / max(abs(valley1.real))
kwant.plotter.density(
    syst=device.fsyst,
    density=valley_density,
    cmap="RdBu_r",
    vmin=-0.5,
    vmax=0.5,
    ax=ax1,
    background="white",
)

from matplotlib import cm
import matplotlib

norm = matplotlib.colors.Normalize(vmin=-1, vmax=1, clip=False)
clb1 = fig.colorbar(
    cm.ScalarMappable(norm=norm, cmap="RdBu_r"), ax=ax1, orientation="vertical", pad=0.1
)
# clb1.set_label(r'$\mathcal{V}(\mathbf{r})$', labelpad = 5)
clb1.set_ticks([-1, 0, 1])
clb1.set_ticklabels([r"$\mathcal{V} < 0$", r"$0$", r"$\mathcal{V} > 0$"])

kwant.plot(device2.fsyst, show=False, ax=ax1, site_color="k")


ax1.set_xlim(-200, 200)
ax1.set_ylim(-30, 630)
# ax1.set_xticks([])
# ax1.set_yticks([])

kwant.plot(device2.fsyst, show=False, ax=ax2, site_color="k")
eh_density = eh1.real / max(abs(eh1.real))
## Plot valley number
kwant.plotter.density(
    syst=device.fsyst,
    density=eh_density * 3,
    cmap="RdBu_r",
    vmin=-0.5,
    vmax=0.5,
    ax=ax2,
    background="white",
)
ax2.set_xlim(-200, 200)
ax2.set_ylim(-30, 630)
# ax2.set_xticks([])
# ax2.set_yticks([])

clb2 = fig.colorbar(
    cm.ScalarMappable(norm=norm, cmap="RdBu_r"), ax=ax2, orientation="vertical", pad=0.1
)
# clb2.set_label(r'$\mathcal{C}(\mathbf{r})$', labelpad = 5)
clb2.set_ticks([-1, 0, 1])
clb2.set_ticklabels([r"$\mathcal{C} < 0$", r"$0$", r"$\mathcal{C} > 0$"])

fig.subplots_adjust(
    left=None, bottom=None, right=None, top=None, wspace=0.1, hspace=None
)

ax1.text(-0.5, 0.9, r"$(a)$", transform=ax1.transAxes)
ax2.text(-0.2, 0.9, r"$(b)$", transform=ax2.transAxes)
ax2.set_yticklabels([])

fig.tight_layout()
fig.savefig("./plots/fig5.pdf")
fig.show()
# -


