# +
import numpy as np
from modules import ns_junction

import matplotlib
import matplotlib.pyplot as plt
from matplotlib import rc

matplotlib.use("pdf")
matplotlib.rcParams.update(
    {
        "pgf.texsystem": "xelatex",
        "font.family": "serif",
        "text.usetex": True,
        "pgf.rcfonts": False,
    }
)
# -

a = ns_junction.a
t = ns_junction.t
mu = np.linspace(-0.14 * t, 0, 50)


# Superconductor covering opposite edges

geo_params_opposite = dict(
    edge_type="zigzag",
    lead=True,
    QH_top_boundary=100.5 * a,
    QH_right_boundary=220 * a,
    QH_bottom_boundary=0.5 * a,
    QH_left_boundary=0 * a,
    SC_top_boundary=100.5 * a,
    SC_right_boundary=220 * a,
    SC_bottom_boundary=0.5 * a,
    SC_left_boundary=121 * a,
)

ham_params_opposite = dict(
    mu_N=-0.1 * t,
    mu_QH=-0.1 * t,
    mu_SC=-0.1 * t,
    phi=1 / 500,
    Delta=0.05 * t,
    smooth_mu_SC=(0, 0, 0, 50 * a),
    smooth_mu_QH=(0, 0, 0, 50 * a),
    smooth_B=(0, 0, 0, 0),
    smooth_Delta=(0, 0, 0, 50 * a),
    smooth_B_SC=(0, 0, 0, 50 * a),
)

JunctionOpposite = ns_junction.Junction(geo_params_opposite)
JunctionOpposite.compute_ns_conductance(mu, ham_params_opposite)


# Superconductor covering adjacent edges

geo_params_adjacent = dict(
    edge_type="zigzag",
    lead=True,
    QH_top_boundary=200 * a,
    QH_right_boundary=320 * a,
    QH_bottom_boundary=0.5 * a,
    QH_left_boundary=0 * a,
    SC_top_boundary=150 * a,
    SC_right_boundary=320 * a,
    SC_bottom_boundary=0.5 * a,
    SC_left_boundary=100 * a,
)

ham_params_adjacent = dict(
    mu_N=-0.4 * t,
    mu_QH=-0.1 * t,
    mu_SC=-0.1 * t,
    phi=1 / 500,
    Delta=0.05 * t,
    smooth_mu_SC=(50 * a, 0, 0, 50 * a),
    smooth_mu_QH=(50 * a, 0, 0, 50 * a),
    smooth_B_SC=(50 * a, 0, 0, 50 * a),
    smooth_B=(0, 0, 0, 0),
    smooth_Delta=(50 * a, 0, 0, 50 * a),
)

JunctionAdjacent = ns_junction.Junction(geo_params_adjacent)
JunctionAdjacent.compute_ns_conductance(mu, ham_params_adjacent)


# Superconductor covering a single edge

geo_params_single = dict(
    edge_type="zigzag",
    lead=True,
    QH_top_boundary=200.5 * a,
    QH_right_boundary=400 * a,
    QH_bottom_boundary=0.5 * a,
    QH_left_boundary=0 * a,
    SC_top_boundary=150 * a,
    SC_right_boundary=300 * a,
    SC_bottom_boundary=0.5 * a,
    SC_left_boundary=81 * a,
)

ham_params_single = dict(
    mu_N=-0.4 * t,
    mu_QH=-0.1 * t,
    mu_SC=-0.1 * t,
    phi=1 / 200,
    Delta=0.05 * t,
    smooth_mu_SC=(50 * a, 50 * a, 0, 50 * a),
    smooth_mu_QH=(0, 0, 0, 20 * a),
    smooth_B_SC=(50 * a, 50 * a, 0, 50 * a),
    smooth_B=(0, 0, 0, 0),
    smooth_Delta=(0, 0, 0, 0),
)

JunctionSingle = ns_junction.Junction(geo_params_single)
JunctionSingle.compute_ns_conductance(mu, ham_params_single)


# Plot conductances

fig, ax = plt.subplots(figsize=(2, 5))
ax.plot(mu, JunctionOpposite.conductance[0], lw=2, color="b")
ax.plot(mu, JunctionAdjacent.conductance[0], lw=2, color="b")
ax.plot(mu, JunctionSingle.conductance[0], lw=2, color="b")
ax.text(-0.003, 2.07, r"$\Phi=\pi$", color="b", fontsize=22)
ax.text(-0.003, 1.2, r"$\Phi=\pi/2$", color="b", fontsize=22)
ax.text(-0.003, 0.07, r"$\Phi=0$", color="b", fontsize=22)
ax.set_ylabel(r"$G_{NS}\ [2e^2/h]$", fontsize=22)
ax.set_xlabel(r"$\mu_{QH}\ [t]$", fontsize=22)
ax.tick_params(axis="both", which="major", labelsize=22)
ax.set_xlim([np.min(mu), np.max(mu)])
ax.set_ylim([-0.2, 2.25])
plt.savefig("./plots/fig6.pdf", bbox_inches="tight")
plt.show()


