from .constants import a_0, t_0
import numpy as np


def rescale(device):
    """
    Set rescaled lattice.

    To reduce computational cost, we perform the calculations
    with a rescaled graphene structure. Both the lattice and hopping
    constants are rescaled to keep the Fermi velocity fixed.
    """

    device.a = a_0 * device.scaling_factor
    device.t = t_0 / device.scaling_factor

    # Lattice constant for the square lattice.
    if device.interface == "armchair":
        device.square_a = 3 / 4 * device.a
    elif device.interface == "zigzag":
        device.square_a = np.sqrt(3) * device.a / 2

    device.geometries = {
        "armchair": {
            "prim_vec": (
                (0, 3 * device.a),
                (-np.sqrt(3) * device.a / 2, 3 * device.a / 2),
            ),
            "basis_vec": ((0, 0), (0, device.a)),
            "sym_period": (1, -2),
            "offset": (0.5 * device.a, -0.5 * device.a),
        },
        "zigzag": {
            "prim_vec": (
                (0, device.a * np.sqrt(3)),
                (-device.a * 3 / 2, device.a * np.sqrt(3) / 2),
            ),
            "basis_vec": ((0, 0), (device.a, 0)),
            "sym_period": (1, -2),
            "offset": (0, 0),
        },
        "square_armchair": {
            "prim_vec": [(device.square_a, 0), (0, device.square_a)],
            "basis_vec": [[0, (2 - 3 / 8) * device.a]],
            "sym_period": (1, 0),
            "offset": (0, 0),
        },
        "square_zigzag": {
            "prim_vec": [(device.square_a, 0), (0, device.square_a)],
            "basis_vec": [[0, (2 - 3 / 8) * device.a]],
            "sym_period": (1, 0),
            "offset": (0, 0),
        },
    }
