from . import *
from . import shapes
import kwant
import math
import numpy as np
from .pauli import tau_0, tau_1, tau_2, tau_3, tau_plus, tau_minus
from .constants import nano, phi_0, mu_B, g

# All energies measured in units of t
t = 1


def smooth_boundary(x, y, l):
    """
    Define a smooth function at the boundaries.
    1 for x -> ∞  (QH region)
    0 for x -> -∞ (SC region)

    Parameters:
    -----------

    x : float
        x coordinate
    y : float
        y coordinate
    """
    return 1 / 2 * (1 + np.tanh(x / l))


def peierls_phase(site1, site2, B):
    """
    Calculate the Peierls phase with Landau gauge.

    Parameters:
    -----------
    r1 : Kwant site
    r2 : Kwant site
    B : float
        magnetic field [T]
    """
    # Extract coordinates and convert to meters
    x1, y1 = site1.pos * nano
    x2, y2 = site2.pos * nano
    # Define mean values
    xmed = (x1 + x2) / 2
    ymed = (y1 + y2) / 2
    # Calculate the gauge field
    gauge = -B * xmed * (1 - np.heaviside(xmed, 1 / 2))
    return np.exp(1j * 2 * np.pi / phi_0 * (y2 - y1) * gauge)


def onsite_scattering(
    site,
    mu,
    mu_SC,
    Delta,
    l,
    delta_interface,
    delta_boundary,
    lB_min,
    W_device,
    L_device,
    L_contact,
    W_lead,
    theta,
    B,
    spinful,
):
    """
    Define onsite energies at the scattering region.

    Parameters:
    -----------

    site : Kwant site
    mu : float
        Chemical potential @ quantum Hall region.
    mu_SC : float
        Chemical potential @ superconducting region.
    Delta : float
        Superconducting order parameter.
    l : float
        Smoothing lenght.
    delta_interface : float
        Interface isorder amplitude.
    delta_boundary : float
        Boundary disorder amplitude.
    lB_min : float
        Magnetic lenght of the lowest field applied.
    W_device : float
        System's width.
    L_device : float
        System's lenght.
    L_contact : float
        Lenght of the superconducting contact.
    W_lead : float
        Width of the normal leads.
    theta : float
        Angle of the superconducting lead.
    B : float
        Magnetic field.
    spinful : bool
        If True, set spinful Hamiltonian.
    """

    def disorder_interface(site):
        """
        Set region for disordered interface.

        Parameters:
        -----------

        site : Kwant site
        """
        x, y = site.pos
        return -lB_min / 5 <= x <= L_contact

    def disorder_boundary(site):
        """
        Set region for boundary disorder.

        Parameters:
        -----------

        site : Kwant site
        """
        x, y = site.pos
        upper = W_device - lB_min / 5 + math.tan(theta) * x
        lower = lB_min / 5 + math.tan(theta) * x

        x_condition = -L_device + W_lead - lB_min / 5 < x < -lB_min / 5
        y_condition = not (lower < y < upper)

        return x_condition and y_condition

    # Extract coordinates
    x, y = site.pos

    # Add interface disorder
    if delta_interface:
        if disorder_interface(site):
            delta_mu_interface = (
                delta_interface
                * 2
                * (kwant.digest.uniform(repr(site.pos)) - 0.5)
                * tau_3
            )
        else:
            delta_mu_interface = 0.0 * tau_3
    else:
        delta_mu_interface = 0.0 * tau_3

    # Add boundary disorder
    if delta_boundary:
        if disorder_boundary(site):
            delta_mu_boundary = (
                delta_boundary
                * 2
                * (kwant.digest.uniform(repr(site.pos)) - 0.5)
                * tau_3
            )
        else:
            delta_mu_boundary = 0.0 * tau_3
    else:
        delta_mu_boundary = 0.0 * tau_3

    # Define chemical potential term
    if not mu_SC:
        mu_SC = mu
    chem_pot = -((mu_SC - mu) * smooth_boundary(x, y, l) + mu) * tau_3

    # Define superconducting term
    Delta_SC = Delta * np.heaviside(x, 0.5) * tau_1

    # Spinless onsite
    onsite = chem_pot + Delta_SC + delta_mu_interface + delta_mu_boundary

    # Spinfull
    if spinful:
        zeeman = 0.5 * g * mu_B * B * (1 - np.heaviside(x, 1 / 2))
        onsite = np.kron(onsite, tau_0) + zeeman * np.kron(tau_0, tau_3)

    return onsite


def onsite_scattering_with_square_latt(
    site,
    mu,
    mu_SC,
    Delta,
    l,
    delta_interface,
    delta_boundary,
    lB_min,
    W_device,
    L_device,
    L_contact,
    W_lead,
    theta,
    B,
    spinful,
):
    """
    Define onsite energies at the scattering region.

    Parameters:
    -----------

    site : Kwant site
    mu : float
        Chemical potential @ quantum Hall region.
    mu_SC : float
        Chemical potential @ superconducting region.
    Delta : float
        Superconducting order parameter.
    l : float
        Smoothing lenght.
    delta_interface : float
        Interface isorder amplitude.
    delta_boundary : float
        Boundary disorder amplitude.
    lB_min : float
        Magnetic lenght of the lowest field applied.
    W_device : float
        System's width.
    L_device : float
        System's lenght.
    L_contact : float
        Lenght of the superconducting contact.
    W_lead : float
        Width of the normal leads.
    theta : float
        Angle of the superconducting lead.
    B : float
        Magnetic field.
    spinful : bool
        If True, set spinful Hamiltonian.
    """
    # Extract coordinates
    x, y = site.pos

    def disorder_interface(site):
        """
        Set region for disordered interface.

        Parameters:
        -----------

        site : Kwant site
        """
        x, y = site.pos
        return -lB_min / 5 <= x <= L_contact

    def disorder_boundary(site):
        """
        Set region for boundary disorder.

        Parameters:
        -----------

        site : Kwant site
        """
        x, y = site.pos
        upper = W_device - lB_min / 5 + math.tan(theta) * x
        lower = lB_min / 5 + math.tan(theta) * x

        x_condition = -L_device + W_lead - lB_min / 5 < x < -lB_min / 5
        y_condition = not (lower < y < upper)

        return x_condition and y_condition

    # Extract coordinates
    x, y = site.pos

    # Add interface disorder
    if delta_interface:
        if disorder_interface(site):
            delta_mu_interface = (
                delta_interface
                * 2
                * (kwant.digest.uniform(repr(site.pos)) - 0.5)
                * tau_3
            )
        else:
            delta_mu_interface = 0.0 * tau_3
    else:
        delta_mu_interface = 0.0 * tau_3

    # Add boundary disorder
    if delta_boundary:
        if disorder_boundary(site):
            delta_mu_boundary = (
                delta_boundary
                * 2
                * (kwant.digest.uniform(repr(site.pos)) - 0.5)
                * tau_3
            )
        else:
            delta_mu_boundary = 0.0 * tau_3
    else:
        delta_mu_boundary = 0.0 * tau_3

    # Define chemical potential term
    if not mu_SC:
        mu_SC = mu
    chem_pot = -((mu_SC - mu) * smooth_boundary(x, y, l) + mu) * tau_3

    # Define superconducting term
    Delta_SC = Delta * np.heaviside(x, 0.5) * tau_1

    # Spinless onsite
    onsite = chem_pot + Delta_SC + delta_mu_interface + delta_mu_boundary

    # Spinfull
    if spinful:
        zeeman = 0.5 * g * mu_B * B * (1 - np.heaviside(x, 1 / 2))
        onsite = np.kron(onsite, tau_0) + zeeman * np.kron(tau_0, tau_3)

    return onsite


def onsite_lead(site, mu, B, spinful):
    """
    Define onsite energies at the scattering region.

    Parameters:
    -----------

    site : Kwant site
    mu : float
        chemical potential @ quantum Hall region
    Delta : float
        superconducting order parameter
    spinful : bool
        If True, set spinful Hamiltonian.
    """
    # Extract coordinates
    x, y = site.pos

    # Spinless
    onsite = -mu * tau_3

    # Spinfull
    if spinful:
        zeeman = 0.5 * g * mu_B * B * (1 - np.heaviside(x, 1 / 2))
        onsite = np.kron(onsite, tau_0) + zeeman * np.kron(tau_0, tau_3)

    return onsite


def onsite_SC_lead(site, mu, mu_SC, Delta_lead, spinful):
    """
    Define onsite energies at the scattering region.

    Parameters:
    -----------

    site : Kwant site
    mu_SC : float
        chemical potential @ superconducting region
    Delta : float
        superconducting order parameter
    spinful : bool
        If True, set spinful Hamiltonian.
    """
    # Extract coordinates
    x, y = site.pos

    # define chemical potential term
    if not mu_SC:
        mu_SC = mu
    chem_pot = -mu_SC * tau_3

    SC = Delta_lead * tau_1

    onsite = chem_pot + SC

    # Spinfull
    if spinful:
        onsite = np.kron(onsite, tau_0)

    return onsite


def onsite_SC_square_lead(site, mu, mu_SC, Delta_lead, spinful):
    """
    Define onsite energies at the scattering region.
    Chosen in accordance with arXiv:1907.01722, part S.6

    Parameters:
    -----------

    site : Kwant site
    mu_SC : float
        chemical potential @ superconducting region
    Delta : float
        superconducting order parameter
    spinful : bool
        If True, set spinful Hamiltonian.
    """
    # Extract coordinates
    x, y = site.pos

    # define chemical potential term
    if not mu_SC:
        mu_SC = mu
    chem_pot = -mu_SC * tau_3
    SC = Delta_lead * tau_1

    onsite = chem_pot + SC

    # Spinfull
    if spinful:
        onsite = np.kron(onsite, tau_0)

    return onsite


# Hoppings are the same for both leads and scattering IF the system has
# translational symmetry along the y-direction
def hopping(site1, site2, B, spinful):
    """
    Define hopping energies.

    Parameters:
    -----------

    site1 : Kwant site
    site2 : Kwant site
    B : float
        magnetic field [T]
    spinful : bool
        If True, set spinful Hamiltonian.
    """
    # Dalculate Peierls phase
    peierls = peierls_phase(site1, site2, B)
    # Define electron hopping
    hopping_e = -t * peierls * tau_plus
    # Define hole hopping
    hopping_h = t * np.conj(peierls) * tau_minus

    hopping = hopping_e + hopping_h

    # Spinfull
    if spinful:
        hopping = np.kron(hopping, tau_0)

    return hopping


def hopping_with_square_latt(site1, site2, B, spinful):
    """
    Define hopping energies.

    Parameters:
    -----------

    site1 : Kwant site
    site2 : Kwant site
    B : float
        magnetic field [T]
    spinful : bool
        If True, set spinful Hamiltonian.
    """

    # calculate Peierls phase
    peierls = peierls_phase(site1, site2, B)
    # hopping potential is different on the square lattice
    x_hop = 0.5 * (site1.pos[0] + site2.pos[0])
    t_func = t - t / 2 * np.heaviside(x_hop, 1 / 2)
    # define electron hopping
    hopping_e = -t_func * peierls * tau_plus
    # define hole hopping
    hopping_h = t_func * np.conj(peierls) * tau_minus

    hopping = hopping_e + hopping_h

    # Spinfull
    if spinful:
        hopping = np.kron(hopping, tau_0)

    return hopping


def hopping_SC(site1, site2, spinful):
    """
    Define hopping energies in the reight lead (honeycomb).

    Parameters:
    -----------

    sote1 : Kwant site
    site2 : Kwant site
    spinful : bool
        If True, set spinful Hamiltonian.
    """
    hopping = -t * tau_3
    # Spinfull
    if spinful:
        hopping = np.kron(hopping, tau_0)

    return hopping


def hopping_SC_square(site1, site2, spinful):
    """
    Define hopping energies in the reight lead (square).
    Chosen in accordance with arXiv:1907.01722, part S.6

    Parameters:
    -----------

    site1 : Kwant site
    site2 : Kwant site
    spinful : bool
        If True, set spinful Hamiltonian.
    """

    hopping = -t / 2 * tau_3

    # Spinfull
    if spinful:
        hopping = np.kron(hopping, tau_0)

    return hopping


def antihaldane(site1, site2, B, spinful):
    """
    Define anti-Haldane coupling.

    Peierls phase corrections are included so the valley
    polarization is gauge invariant.

    Parameters:
    -----------

    site1 : Kwant site
    site2 : Kwant site
    B : float
        magnetic field [T]
    spinful : bool
        If True, set spinful Hamiltonian.
    """
    # calculate Peierls phase
    peierls = peierls_phase(site1, site2, B)
    # define electron anti-Haldane
    antihaldane_e = 1j / (3 * np.sqrt(3)) * peierls * tau_plus
    # define hole anti-Haldane
    antihaldane_h = -1j / (3 * np.sqrt(3)) * np.conj(peierls) * tau_minus

    antihaldane = antihaldane_e + antihaldane_h

    # Spinfull
    if spinful:
        antihaldane = np.kron(antihaldane, tau_0)

    return antihaldane


def onsite_infinite(site, mu, mu_SC, Delta, l, B, spinful):
    """
    Define onsite energies at the scattering region.

    Parameters:
    -----------

    site : Kwant site
    mu : float
        Chemical potential @ quantum Hall region.
    mu_SC : float
        Chemical potential @ superconducting region.
    Delta : float
        Superconducting order parameter.
    B : float
        Magnetic field.
    spinful : bool
        If True, set spinful Hamiltonian.
    """
    # Extract coordinates
    x, y = site.pos
    # Define chemical potential term
    if not mu_SC:
        mu_SC = mu
    chem_pot = -((mu_SC - mu) * smooth_boundary(x, y, l) + mu) * tau_3
    # Define superconducting term
    Delta_SC = Delta * np.heaviside(x, 0.5) * tau_1

    onsite = chem_pot + Delta_SC

    # Spinfull
    if spinful:
        zeeman = 0.5 * g * mu_B * B * (1 - np.heaviside(x, 1 / 2))
        onsite = np.kron(onsite, tau_0) + zeeman * np.kron(tau_0, tau_3)

    return onsite


def create_squareinterface_hoppings(device):
    """
    Creates hopping links between a honeycomb lattice and a square lattice,
    given a device that already has a system object comprised of 3 sublattices
    A, B, C of honeycomb lattice (A, B) and square lattice (C).

    Parameters:
    -----------

    device: instance of Device
        Contains a kwant.Builder() instance `syst`
    """

    sites = device.syst.sites()
    honey_boundary_sites = []
    honey_y = []
    square_boundary_sites = []

    # horizontal interval containing the sites at the armchair
    # NS interface:
    honeycomb_interval = [-device.a * 2 - 1e-4 * device.a, -device.a * 1e-6]
    # horizontal interval containing the sites at the square
    # lattice NS interface:
    square_interval = [0, 0.5 * device.square_a]

    # Store the sites that are within these intervals in arrays:
    for site in sites:
        if honeycomb_interval[0] <= site.pos[0] <= honeycomb_interval[1]:
            honey_boundary_sites.append(site)
            # store the y-coordinate of the honeycomb sites
            honey_y.append(site.pos[1])
        elif square_interval[0] <= site.pos[0] <= square_interval[1]:
            square_boundary_sites.append(site)

    pairs = []
    honey_y = np.array(honey_y)

    # For each site on the square lattice NS interface, find the
    # site on the honeycomb structure that has the closest
    # y-coordinate, and store them as a pair:
    for i, r_site in enumerate(square_boundary_sites):
        y_r = r_site.pos[1]
        index = np.argmin(np.abs(honey_y - y_r))
        pairs.append([r_site, honey_boundary_sites[index]])

    if device.spinful:
        hop = -t * np.kron(tau_3, tau_0)
    else:
        hop = -t * tau_3

    # Create hopping between pairs:
    for i, pair in enumerate(pairs):
        # Like in the pre-print, hopping at the interface taken
        # same as in scattering region
        if int(pair[1].family.name) == 0:
            # Sublattice A <-> Square Lattice
            device.syst[device.C(*pair[0].tag), device.A(*pair[1].tag)] = hop
        else:
            # Sublattice B <-> Square Lattice
            device.syst[device.C(*pair[0].tag), device.B(*pair[1].tag)] = hop
