from . import *
from . import functions, shapes, crystal, system
import kwant
from functools import partial
from .pauli import tau_2, tau_3, tau_0


class Device:
    def __init__(self, dimensions, scaling_factor, interface="zigzag", spinful=False):
        """
        Initialization function.

        Parameters:
        -----------

        dimensions : dict
            System's dimensions.
        Interface : string
            NS interface ('zigzag' or 'armchair').
        scaling_factor
            Scaling factor (transforms a -> s * a and t -> t / s).
        spinful : bool
            If True, set spinful system.
        """

        self.scaling_factor = scaling_factor
        self.interface = interface
        crystal.rescale(self)
        self.geometry = self.geometries[interface]
        self.dimensions = dimensions
        self.spinful = spinful

        print("-----------------------------------------------------------")
        print("New lattice constant is " + str(round(self.a, 2)) + "nm.")
        print("New hopping constant is " + str(round(self.t, 2)) + "eV")
        print("-----------------------------------------------------------")

    def make_bulk_honeymcomb(self, onsite, hopping):
        """
        Make bulk graphene.

        Parameters:
        -----------

        onsite : float or function
            Onsite energy.
        hopping : float or function
            Hopping energy.
        """

        # Create bulk Builder
        bulk = kwant.Builder(self.honeycomb_sym)

        # Set onsite energy for subattice A
        # bulk[self.A(0, 0)] = onsite_dev
        bulk[self.A(0, 0)] = onsite
        # Set onsite energy for sublattice B
        # bulk[self.B(0, 0)] = onsite_dev
        bulk[self.B(0, 0)] = onsite
        # Set hopping energy
        bulk[self.lattice.neighbors()] = hopping

        return bulk

    def make_square_bulk(self, onsite, hopping):
        """
        Make square bulk for the SC region.

        Parameters:
        -----------

        onsite : float or function
            Onsite energy.
        hopping : float or function
            Hopping energy.
        """

        # Create bulk Builder
        bulk = kwant.Builder(self.square_sym)
        # Set onsite energy
        bulk[self.C(0, 0)] = onsite
        # Set hopping energy
        bulk[self.square_lattice.neighbors()] = hopping

        return bulk

    def translational_invariant_system_y(self, onsite, hopping, shape):
        """
        Create and attach upper and lower leads.

        Parameters:
        -----------

        syst : Kwant builder
        onsite : float or function
            Onsite energy in the leads.
        hopping : float or functions
            Hopping energy in the leads.
        shape : function
            Shape of upper and lower leads.
        """

        # Create bulk Builder upper and lower leads
        lead_bulk = self.make_bulk_honeymcomb(onsite, hopping)
        # Define translational symmetry
        infinite_system_sym = self.honeycomb_sym.subgroup((1, 0))

        # Create lead Builder
        infinite_system = kwant.Builder(infinite_system_sym)

        # Fill lead Builder
        added_sites = infinite_system.fill(
            lead_bulk, lambda site: shape(site, self), (0, 0), max_sites=float("inf")
        )

        # Remove dangling bonds
        infinite_system.eradicate_dangling()

        return infinite_system

    def translational_invariant_system_y_square(self, onsite, hopping):
        """
        Create and attach upper and lower leads.

        Parameters:
        -----------

        syst : Kwant builder
        onsite : float or function
            Onsite energy in the leads.
        hopping : float or functions
            Hopping energy in the leads.
        """

        # Define translational symmetry: vector commensurate with lead period
        infinite_sys_sym = self.honeycomb_sym.subgroup((1, 0))

        # Create lead Builder
        infinite_system = kwant.Builder(infinite_sys_sym)

        # Create template builders
        scattering_honeycomb_bulk = self.make_bulk_honeymcomb(onsite, hopping)
        scattering_square_bulk = self.make_square_bulk(onsite, hopping)

        # Fill the templates
        scattering_region_honeycomb_sites = infinite_system.fill(
            scattering_honeycomb_bulk,
            lambda site: shapes.infinite_system_shape_QHregion_with_square(site, self),
            (0, 0),
            max_sites=float("inf"),
        )
        scattering_region_square_sites = infinite_system.fill(
            scattering_square_bulk,
            lambda site: shapes.infinite_system_shape_SCregion_with_square(site, self),
            (0, 0),
            max_sites=float("inf"),
        )

        return infinite_system

    def make_system(self, SC_square_bool=False):

        """
        Make Builder (unfinalized system).
        """

        if self.spinful:
            self.norbs = 4
        else:
            self.norbs = 2

        # Honeycomb structure
        off = self.geometry["offset"]
        ## Lattice
        self.lattice = kwant.lattice.Polyatomic(
            self.geometry["prim_vec"], self.geometry["basis_vec"], norbs=self.norbs
        )
        self.honeycomb_sym = kwant.TranslationalSymmetry(
            self.lattice.vec((1, 0)), self.lattice.vec((0, 1))
        )
        ## Sublattices
        self.A, self.B = self.lattice.sublattices

        if SC_square_bool:
            # Square structure
            ## Lattice
            square_interface = "square_" + self.interface
            self.square_lattice = kwant.lattice.general(
                self.geometries[square_interface]["prim_vec"],
                self.geometries[square_interface]["basis_vec"],
                norbs=self.norbs,
            )
            ## Bulk symmetry
            self.square_sym = kwant.TranslationalSymmetry(
                self.square_lattice.vec((1, 0)), self.square_lattice.vec((0, 1))
            )

            ## Sites family
            self.C = self.square_lattice.sublattices[0]

            # Create lead
            self.syst = self.translational_invariant_system_y_square(
                onsite=functions.onsite_infinite,
                hopping=functions.hopping_with_square_latt,
            )

            # Create hoppings between square SC and honeycomb QH
            functions.create_squareinterface_hoppings(self)

        else:
            # Create lead
            self.syst = self.translational_invariant_system_y(
                onsite=functions.onsite_infinite,
                hopping=functions.hopping,
                shape=shapes.infinite_system_shape,
            )

        self.fsyst = self.syst.finalized()
