import numpy as np
import kwant
from .pauli import tau_0, tau_1, tau_3
from .constants import a_0, t_0

# no rescaling:
scaling_factor = 100
a = a_0 * scaling_factor
t = t_0 / scaling_factor

geometries = dict(
    zigzag=dict(
        prim_vec=((0, 3 * a), (-np.sqrt(3) * a / 2, 3 * a / 2)),
        basis_vec=((0, 0), (0, a)),
        sym_period=(1, -2),
    ),
    armchair=dict(
        prim_vec=((0, a * np.sqrt(3)), (-a * 3 / 2, a * np.sqrt(3) / 2)),
        basis_vec=((0, 0), (a, 0)),
        sym_period=(1, -2),
    ),
)


class Junction:
    def __init__(self, geo_params):
        """Stores system geometry parameters."""
        for key, value in geo_params.items():
            setattr(self, key, value)

    def build_system(self):
        """Makes Kwant Builder."""

        def region_N(pos):
            """Shape of the normal metal region."""
            return self.QH_bottom_boundary <= pos[1] <= self.QH_top_boundary

        def region_QH(pos):
            """Shape of the QH region."""
            return (
                self.QH_left_boundary <= pos[0] <= self.QH_right_boundary
                and self.QH_bottom_boundary <= pos[1] <= self.QH_top_boundary
            )

        def region_SC(pos):
            """Shape of the SC region."""
            return (
                self.SC_left_boundary <= pos[0] <= self.SC_right_boundary
                and self.SC_bottom_boundary <= pos[1] <= self.SC_top_boundary
            )

        def onsite_N(site, mu_N):
            """Onsite potential in the normal metal."""
            return mu_N * tau_3

        def onsite_QH(site, mu_N, mu_QH, smooth_mu_QH):
            """Onsite potential in the QH region."""
            # Chemical potential smoothing between N region and QH region:
            mu_z = smooth_tanh_2D(site.pos, r0_QH, (mu_N, mu_QH), smooth_mu_QH)
            return mu_z * tau_3

        def onsite_SC(site, mu_QH, mu_SC, Delta, smooth_mu_SC, smooth_Delta):
            """Onsite potential in the SC region."""
            # Chemical potential smoothing between QH region and SC region:
            mu_z = smooth_tanh_2D(site.pos, r0_SC, (mu_QH, mu_SC), smooth_mu_SC)
            # Superconducting gap smoothing between QH region and SC region:
            mu_x = smooth_tanh_2D(site.pos, r0_SC, (0, Delta), smooth_Delta)
            return mu_z * tau_3 + mu_x * tau_1

        def hopping_N(site1, site2):
            """Hopping potential in the normal metal."""
            return -t * tau_3

        def hopping_QHSC(site1, site2, phi, smooth_B, smooth_B_SC):
            """Hopping potentials in the QH region and in the SC region."""
            x1, y1 = site1.pos
            x2, y2 = site2.pos
            area = 3 / 2 * np.sqrt(3) * a ** 2
            theta = 2 * np.pi * 1 / 2 * phi * (x1 - x2) * (y1 + y2) / area
            not_SC = not (region_SC(site1.pos) or region_SC(site2.pos))
            # Magnetic field smoothing between QH region and SC region:
            amplitude = smooth_tanh_2D(
                np.mean([site1.pos, site2.pos], axis=0),
                r0_QH if not_SC else r0_SC,
                (0, 1) if not_SC else (1, 0),
                smooth_B if not_SC else smooth_B_SC,
            )
            return -t * (
                np.cos(amplitude * theta) * tau_3
                + 1j * np.sin(amplitude * theta) * tau_0
            )

        def conservation_func(site):
            """Conservation law in the QH region."""
            return -tau_3 if (region_QH(site.pos)) else None

        r0_QH = (
            self.QH_top_boundary,
            self.QH_right_boundary,
            self.QH_bottom_boundary,
            self.QH_left_boundary,
        )
        r0_SC = (
            self.SC_top_boundary,
            self.SC_right_boundary,
            self.SC_bottom_boundary,
            self.SC_left_boundary,
        )

        geometry = geometries[self.edge_type]
        lattice = kwant.lattice.Polyatomic(
            geometry["prim_vec"], geometry["basis_vec"], norbs=2
        )
        sym = kwant.TranslationalSymmetry(lattice.vec(geometry["sym_period"]))
        syst = kwant.Builder(conservation_law=conservation_func)

        # Build QH and SC regions
        syst[
            lattice.shape(region_QH, (self.QH_left_boundary, self.QH_bottom_boundary))
        ] = onsite_QH
        syst[
            lattice.shape(region_SC, (self.SC_left_boundary, self.SC_bottom_boundary))
        ] = onsite_SC
        syst[lattice.neighbors()] = hopping_QHSC

        # Build incoming normal metal lead
        lead = kwant.Builder(sym, conservation_law=-tau_3)
        lead[
            lattice.shape(region_N, (self.QH_left_boundary, self.QH_bottom_boundary))
        ] = onsite_N
        lead[lattice.neighbors()] = hopping_N

        syst.attach_lead(lead.reversed())
        self.syst = syst

    def compute_ns_conductance(self, mus, params):
        """Computes Smatrix of NS junction."""

        try:
            self.syst
        except AttributeError:
            self.build_system()

        conductance = []
        for i, mu in enumerate(mus):
            for var in ["mu_QH", "mu_SC"]:
                params[var] = mu
            smatrix = kwant.smatrix(self.syst.finalized(), 0.0, params=params)
            G = (
                smatrix.submatrix((0, 0), (0, 0)).shape[0]
                + smatrix.transmission((0, 1), (0, 0))
                - smatrix.transmission((0, 0), (0, 0))
            )
            conductance.append(G)

        self.conductance = [np.array(conductance), mus]


def smooth_tanh(pos, x0, y, dx=50 * a):
    """Returns a smoothing tanh function."""
    y_left, y_right = y
    dx = dx / 2
    x1 = x0 + dx / 2
    if x0 <= pos < x0 + 2 * dx:
        ons_pot = y_left - 1 / 2 * (
            np.tanh(2 * (pos - x1 - 0.5 * dx) / (x1 - x0)) + 1
        ) * (y_left - y_right)
    elif x0 + 2 * dx <= pos:
        ons_pot = y_right
    else:
        ons_pot = y_left
    return ons_pot


def smooth_tanh_2D(pos, r0, z, dr):
    """Returns a tanh function in a 2D region with rounded corners."""
    x, y = pos
    b_top, b_right, b_bottom, b_left = r0
    dr_top, dr_right, dr_bottom, dr_left = dr

    if b_left <= x <= b_left + dr_left and b_bottom + dr_bottom <= y <= b_top - dr_top:
        # left boundary
        val = smooth_tanh(x, b_left, z, dr_left)

    elif (
        b_right - dr_right <= x <= b_right
        and b_bottom + dr_bottom <= y <= b_top - dr_top
    ):
        # right boundary
        val = smooth_tanh(dr_right - x, -(b_right - dr_right), z, dr_right)

    elif (
        b_left + dr_left <= x <= b_right - dr_right
        and b_bottom <= y <= b_bottom + dr_bottom
    ):
        # bottom boundary
        val = smooth_tanh(y, b_bottom, z, dr_bottom)

    elif b_left + dr_left <= x <= b_right - dr_right and b_top - dr_top <= y <= b_top:
        # top boundary
        val = smooth_tanh(dr_top - y, -(b_top - dr_top), z, dr_top)

    elif b_left <= x <= b_left + dr_left and b_bottom <= y <= b_bottom + dr_bottom:
        # bottom-left corner
        xs = b_left + dr_left - x
        ys = b_bottom + dr_bottom - y
        theta = np.arctan2(ys, xs)
        dR = (
            dr_bottom
            * dr_left
            / np.sqrt((dr_left * np.sin(theta)) ** 2 + (dr_bottom * np.cos(theta)) ** 2)
        )
        R = np.sqrt((xs) ** 2 + (ys) ** 2)
        val = smooth_tanh(dR - R, 0, z, dR)

    elif b_left <= x <= b_left + dr_left and b_top - dr_top <= y <= b_top:
        # top-left corner
        xs = b_left + dr_left - x
        ys = b_top - dr_top - y
        theta = np.arctan2(ys, xs)
        dR = (
            dr_top
            * dr_left
            / np.sqrt((dr_left * np.sin(theta)) ** 2 + (dr_top * np.cos(theta)) ** 2)
        )
        R = np.sqrt((xs) ** 2 + (ys) ** 2)
        val = smooth_tanh(dR - R, 0, z, dR)

    elif b_right - dr_right <= x <= b_right and b_top - dr_top <= y <= b_top:
        # top-right corner
        xs = b_right - dr_right - x
        ys = b_top - dr_top - y
        theta = np.arctan2(ys, xs)
        dR = (
            dr_top
            * dr_right
            / np.sqrt((dr_right * np.sin(theta)) ** 2 + (dr_top * np.cos(theta)) ** 2)
        )
        R = np.sqrt((xs) ** 2 + (ys) ** 2)
        val = smooth_tanh(dR - R, 0, z, dR)

    elif b_right - dr_right <= x <= b_right and b_bottom <= y <= b_bottom + dr_bottom:
        # bottom-right corner
        xs = b_right - dr_right - x
        ys = b_bottom + dr_bottom - y
        theta = np.arctan2(ys, xs)
        dR = (
            dr_bottom
            * dr_right
            / np.sqrt(
                (dr_right * np.sin(theta)) ** 2 + (dr_bottom * np.cos(theta)) ** 2
            )
        )
        R = np.sqrt((xs) ** 2 + (ys) ** 2)
        val = smooth_tanh(dR - R, 0, z, dR)

    elif (
        b_left + dr_left <= x <= b_right - dr_right
        and b_bottom + dr_bottom <= y <= b_top - dr_top
    ):
        # middle region
        val = z[1]
    else:
        # surrounding region
        val = z[0]
    return val
