from . import *
import math


def device_shape_honeycomb(site, device):
    """
    Define the shape in the honeycomb structure region of the
    device.

    Parameters:
    -----------

    site : Kwant site
    """

    x, y = site.pos
    theta = device.dimensions["theta"]
    upper = device.dimensions["W_device"] + math.tan(theta) * x
    lower = -0.7 * device.a + math.tan(theta) * x
    return lower <= y < upper and -device.dimensions["L_device"] <= x < 0


def device_shape_square(site, device):
    """
    Define the shape in the square lattice region of the device.

    Parameters:
    -----------

    site : Kwant site
    """

    x, y = site.pos

    return (
        -0.7 * device.a <= y < device.dimensions["W_device"]
        and 0 <= x < device.dimensions["L_contact"]
    )


def device_shape(site, device):
    """
    Set device shape.

    Parameters:
    -----------

    site : Kwant site
    """

    x, y = site.pos
    if x <= 0:
        theta = device.dimensions["theta"]
        upper = device.dimensions["W_device"] + math.tan(theta) * x
        lower = -0.7 * device.a + math.tan(theta) * x
    else:
        upper = device.dimensions["W_device"]
        lower = -0.7 * device.a
    return (
        lower <= y < upper
        and -device.dimensions["L_device"] <= x < device.dimensions["L_contact"]
    )


def lead_shape(site, device):
    """
    Set metallic leads shape.

    Parameters:
    -----------

    site : Kwant site
    """

    x, y = site.pos

    return (
        -device.dimensions["L_device"]
        <= x
        < -device.dimensions["L_device"] + device.dimensions["W_lead"]
    )


def SC_lead_shape(site, device):
    """
    Set SC lead shape.

    Parameters:
    -----------

    site : Kwant site
    """

    x, y = site.pos

    return -0.7 * device.a <= y < device.dimensions["W_device"]


def infinite_system_shape(site, device):
    """
    Set translational invariant system along the y-direction.

    Parameters:
    -----------

    site : Kwant site
    device : system
    """
    x, y = site.pos
    return -device.dimensions["width"] / 2 < x < device.dimensions["width"] / 2


def infinite_system_shape_QHregion_with_square(site, device):
    """
    This function defines the range of the left part of the QH-SC interface,
    which is made of a honeycomb structure.

    Parameters:
    -----------
    site : Kwant site
    """

    x, y = site.pos

    return -device.dimensions["width"] / 2 <= x <= 0


def infinite_system_shape_SCregion_with_square(site, device):
    """
    This function defines the range of the right part of the QH-SC interface,
    which is made of a square lattice SC.

    Parameters:
    -----------
    site : Kwant site
    """

    x, y = site.pos

    return 0 <= x <= device.dimensions["width"] / 2
