from . import *
from . import functions, shapes, crystal
import kwant
from functools import partial
from .pauli import tau_2, tau_3, tau_0


class Device:
    def __init__(self, dimensions, scaling_factor, interface="zigzag", spinful=False):
        """
        Initialization function.

        Parameters:
        -----------

        dimensions : dict
            System's dimensions.
        Interface : string
            NS interface ('zigzag' or 'armchair').
        scaling_factor
            Scaling factor (transforms a -> s * a and t -> t / s).
        spinful : bool
            If True, set spinful system.
        """
        self.scaling_factor = scaling_factor
        self.interface = interface
        crystal.rescale(self)
        self.geometry = self.geometries[interface]
        self.dimensions = dimensions
        self.spinful = spinful

        print("-----------------------------------------------------------")
        print("New lattice constant is " + str(round(self.a, 2)) + "nm.")
        print("New hopping constant is " + str(round(self.t, 2)) + "eV")
        print("-----------------------------------------------------------")

    def make_bulk_honeymcomb(self, onsite, hopping):
        """
        Make bulk graphene.

        Parameters:
        -----------

        onsite : float or function
            Onsite energy.
        hopping : float or function
            Hopping energy.
        """

        # Create bulk Builder
        bulk = kwant.Builder(self.honeycomb_sym)

        # Set onsite energy for subattice A
        # bulk[self.A(0, 0)] = onsite_dev
        bulk[self.A(0, 0)] = onsite
        # Set onsite energy for sublattice B
        # bulk[self.B(0, 0)] = onsite_dev
        bulk[self.B(0, 0)] = onsite
        # Set hopping energy
        bulk[self.lattice.neighbors()] = hopping

        return bulk

    def make_square_bulk(self, onsite, hopping):
        """
        Make square bulk for the SC region.

        Parameters:
        -----------

        onsite : float or function
            Onsite energy.
        hopping : float or function
            Hopping energy.
        """

        # Create bulk Builder
        bulk = kwant.Builder(self.square_sym)
        # Set onsite energy
        bulk[self.C(0, 0)] = onsite
        # Set hopping energy
        bulk[self.square_lattice.neighbors()] = hopping

        return bulk

    def fill_scattering_honeycomb(self, syst, onsite, hopping, shape):
        """
        Fill scattering region for full honeycomb.

        syst : Kwant builder
        onsite : float or functions
            Onsite energy in the scattering region.
        hopping : float or function
            Hoppping energy in the scattering region.
        shape : function
            Shape of scattering region.
        """
        # Create bulk Builder for the scattering region
        scattering_bulk = self.make_bulk_honeymcomb(onsite, hopping)

        # Fill the template builder
        scattering_region_sites = syst.fill(
            scattering_bulk,
            lambda site: shape(site, self),
            (0, 0),
            max_sites=float("inf"),
        )

    def fill_scattering_square(
        self, syst, onsite, hopping, shape_square, shape_honeycomb
    ):
        """
        Fill scattering region for honeycomb + square.

        syst : Kwant builder
        onsite : float or functions
            Onsite energy on the scattering region.
        hopping : float or function
            Hopping energy on the scattering region.
        shape_square : function
            Shape of scattering region with square lattice.
        shape_honeycomb : functions
            Shape of scattering region with honeycomb lattice.
        """

        # Create template builders
        scattering_honeycomb_bulk = self.make_bulk_honeymcomb(onsite, hopping)
        scattering_square_bulk = self.make_square_bulk(onsite, hopping)

        # Fill the templates
        scattering_region_honeycomb_sites = syst.fill(
            scattering_honeycomb_bulk,
            lambda site: shape_honeycomb(site, self),
            (-self.dimensions["L_device"] / 2, 0),
            max_sites=float("inf"),
        )

        scattering_region_square_sites = syst.fill(
            scattering_square_bulk,
            lambda site: shape_square(site, self),
            (self.dimensions["L_contact"] / 2, 0),
            max_sites=float("inf"),
        )

        ## Create hoppings between honeycomb structure and square
        ## lattice
        self.syst = syst
        functions.create_squareinterface_hoppings(self)

    def add_upper_lower_leads(self, syst, onsite, hopping, shape):
        """
        Create and attach upper and lower leads.

        Parameters:
        -----------

        syst : Kwant builder
        onsite : float or function
            Onsite energy in the leads.
        hopping : float or functions
            Hopping energy in the leads.
        shape : function
            Shape of upper and lower leads.
        """
        if self.spinful:
            conservation = -np.kron(tau_3, tau_0)
            PH = np.kron(tau_2, tau_0)
        else:
            conservation = -tau_3
            PH = tau_2

        # Upper lead
        # Create bulk Builder upper and lower leads
        lead_bulk = self.make_bulk_honeymcomb(onsite, hopping)
        # Define translational symmetry
        upper_lead_sym = self.honeycomb_sym.subgroup((1, 0))
        # Create lead Builder
        upper_lead = kwant.Builder(
            upper_lead_sym, conservation_law=conservation, particle_hole=PH
        )
        # Fill lead Builder
        added_sites = upper_lead.fill(
            lead_bulk,
            lambda site: shape(site, self),
            (-self.dimensions["L_device"] + self.dimensions["W_lead"] / 2, 0),
            max_sites=float("inf"),
        )
        # Remove dangling bonds
        upper_lead.eradicate_dangling()
        # Attach upper lead
        syst.attach_lead(upper_lead, add_cells=0)
        # Attach lower lead
        syst.attach_lead(upper_lead.reversed(), add_cells=0)

    def add_right_lead_honeycomb(self, syst, onsite, hopping, shape):
        """
        Create and attach right lead (honeycomb).

        syst : Kwant builder
        onsite : float or function
            Onsite energy in the right lead.
        hopping : float or function
            Hopping energy in the lead.
        shape : function
            Shape of right lead.
        """
        # Create bulk builder for right lead
        lead_SC_bulk = self.make_bulk_honeymcomb(onsite, hopping)
        # Translational symmetry of the lead
        SC_lead_sym = self.honeycomb_sym.subgroup((1, -2))
        # Create Builder for the lead
        SC_lead = kwant.Builder(SC_lead_sym)
        # Fill lead
        added_sites = SC_lead.fill(
            lead_SC_bulk,
            lambda site: shape(site, self),
            (-self.dimensions["L_device"] / 2, 50),
            max_sites=float("inf"),
        )
        # Remove dangling bonds
        SC_lead.eradicate_dangling()
        # Attach lead
        syst.attach_lead(SC_lead, add_cells=0)

    def add_right_lead_square(self, syst, onsite, hopping, shape):
        """
        Create and attach right lead (square).

        Parameters:
        -----------

        syst : Kwant builder
        onsite : float or function
            Onsite energy in the right lead.
        hopping : float or function
            Hopping energy in the right lead.
        shape : function
            Shape of the right lead.
        """
        # Creae bulk Builder
        lead_SC_bulk = self.make_square_bulk(onsite=onsite, hopping=hopping)
        # Define translational symmetry for the lead
        square_interface = "square_" + self.interface
        SC_lead_sym = self.square_sym.subgroup(
            self.geometries[square_interface]["sym_period"]
        )
        # Create Builder for the lead
        SC_lead = kwant.Builder(SC_lead_sym)
        # Fill lead
        added_sites = SC_lead.fill(
            lead_SC_bulk, lambda site: shape(site, self), (0, 0), max_sites=float("inf")
        )
        # Attach lead
        syst.attach_lead(SC_lead, add_cells=0)

    def make_system(self, SC_square_bool=False, leads=True, no_dangling=True):

        """
        Make Builder (unfinalized system).

        Parameters:
        -----------

        SC_lead_bool : bool
            if 'True', create SC lead
        """
        if self.spinful:
            norbs = 4
        else:
            norbs = 2

        # Honeycomb structure
        off = self.geometry["offset"]
        ## Lattice
        self.lattice = kwant.lattice.Polyatomic(
            self.geometry["prim_vec"], self.geometry["basis_vec"], norbs=norbs
        )
        ## Bulk symmetry
        self.honeycomb_sym = kwant.TranslationalSymmetry(
            self.lattice.vec((1, 0)), self.lattice.vec((0, 1))
        )
        ## Sublattices
        self.A, self.B = self.lattice.sublattices

        # If applied, square lattice for the superconducting region
        if SC_square_bool:
            square_interface = "square_" + self.interface
            # Square structure
            ## Lattice
            self.square_lattice = kwant.lattice.general(
                self.geometries[square_interface]["prim_vec"],
                self.geometries[square_interface]["basis_vec"],
                norbs=norbs,
            )
            ## Translational symmetry
            self.square_sym = kwant.TranslationalSymmetry(
                self.square_lattice.vec((1, 0)), self.square_lattice.vec((0, 1))
            )
            ## Sites family
            self.C = self.square_lattice.sublattices[0]

        # Create system
        syst = kwant.Builder()

        # Scattering region
        if SC_square_bool:
            ## Fill system for honeycomb + square
            self.fill_scattering_square(
                syst=syst,
                onsite=functions.onsite_scattering_with_square_latt,
                hopping=functions.hopping_with_square_latt,
                shape_square=shapes.device_shape_square,
                shape_honeycomb=shapes.device_shape_honeycomb,
            )
        else:
            ## Fill system with honeycomb only
            self.fill_scattering_honeycomb(
                syst=syst,
                onsite=functions.onsite_scattering,
                hopping=functions.hopping,
                shape=shapes.device_shape,
            )

        # Remove dangling bonds
        if no_dangling:
            syst.eradicate_dangling()

        if leads:
            # Create and attach upper and lower leads
            self.add_upper_lower_leads(
                syst=syst,
                onsite=functions.onsite_lead,
                hopping=functions.hopping,
                shape=shapes.lead_shape,
            )

            # Right lead
            # Create bulk Builder for SC leads
            # Square lattice SC lead
            if SC_square_bool:
                self.add_right_lead_square(
                    syst=syst,
                    onsite=functions.onsite_SC_square_lead,
                    hopping=functions.hopping_SC_square,
                    shape=shapes.SC_lead_shape,
                )

            # Honeycomb SC lead
            else:
                self.add_right_lead_honeycomb(
                    syst=syst,
                    onsite=functions.onsite_SC_lead,
                    hopping=functions.hopping_SC,
                    shape=shapes.SC_lead_shape,
                )

        self.syst = syst
        self.fsyst = syst.finalized()
