from . import *
import kwant
from itertools import product
import numpy as np
import xarray as xr


def transport(device, params, quantity="G_NS"):
    """
    Compute Hall conductance.

    Parameters:
    -----------

    params : dict
        System parameters.
    quantity : string
        Quantity to be computed.
    """

    if params["mu_SC"] is None:
        params["mu_SC"] = params["mu"]

    # compute the S-matrix at zero energy
    smatrix = kwant.smatrix(
        device.fsyst, energy=0.0, params=params, in_leads=[1], out_leads=[0]
    )
    if quantity == "G_xy":
        # compute the electron transmission from
        # lead 0 to lead 1 (Hall conductance)
        value = smatrix.transmission((1, 0), (0, 0))
    elif quantity == "R_xx":
        # compute R_xx resistance
        # (description in the notebook)
        ## Number of modes
        N = smatrix.submatrix((0, 0), (0, 0)).shape[0]
        ## Probability of outgoing electron from
        ## superconducting contact
        Pe = smatrix.transmission((0, 0), (1, 0))
        ## Probability of ougoing hole (Andreev) from
        ## superconducting contact
        Ph = smatrix.transmission((0, 1), (1, 0))
        ## Hall conductance
        Gxy = smatrix.transmission((1, 0), (0, 0))
        ## Transmission from superconducting contact
        ## to metallic electrode
        PNS = N - (Pe - Ph)
        ## R_xx resistance
        value = (Pe - Ph) / (PNS * Gxy)
    elif quantity == "G_NS":
        ## Probability of outgoing electron from
        ## superconducting contact
        Tee = smatrix.transmission((0, 0), (1, 0))
        ## Probability of ougoing hole (Andreev) from
        ## superconducting contact
        Teh = smatrix.transmission((0, 1), (1, 0))
        ## GNS conductance
        value = Tee - Teh
    else:
        print("Invalid quantity. Choose G_xy, R_xx or G_NS.")

    return value


def plot_GNS(
    dataset,
    ax=None,
    fig=None,
    show_y_label=True,
    show_clb=True,
    save=False,
    set_fonts=True,
    vmin=-2,
    vmax=2,
):
    """
    Plot conductance.

    Parameters:
    -----------

    mus : array
        Onsite energies in the quantum Hall region.
    Bs : array
        Magnetic fields.
    """
    import matplotlib.pyplot as plt
    from matplotlib import rc

    if set_fonts:
        rc("font", **{"family": "sans-serif", "sans-serif": ["Helvetica"]})
        rc("text", usetex=True)
        plt.rcParams["figure.figsize"] = (8, 6)
        plt.rcParams["lines.linewidth"] = 0.65
        plt.rcParams["font.size"] = 25
        plt.rcParams["legend.fontsize"] = 25

    if ax:
        ax = ax
    else:
        fig, ax = plt.subplots()

    im = dataset.GNS.plot.imshow(
        x="Bs",
        y="mus",
        cmap="RdBu_r",
        vmin=vmin,
        vmax=vmax,
        interpolation="bilinear",
        add_colorbar=False,
        ax=ax,
    )
    if show_y_label:
        ax.set_ylabel(r"$\mu_{QH}\ [t]$", labelpad=20)
    else:
        ax.set_ylabel(r"", labelpad=0)
    ax.set_xlabel(r"$B\ [T]$", labelpad=20)
    if show_clb:
        clb = fig.colorbar(im, ax=ax)
        clb.set_label(r"$G_{xx}\ \left[ \frac{e^2}{h} \right]$", labelpad=0)
        clb.set_ticks([vmin, vmin / 2, 0, vmax, vmax / 2])
    plt.tight_layout()
    ax.set_xticks([1, 2, 3, 4, 5, 6])
    if save:
        plt.savefig("./plots/GNS_mu_B_" + dataset.attrs["name"] + ".png")
        plt.show()
    else:
        return im


def GNS_mu_B(mus, Bs, params, device, lview):
    """
    Plot G_NS.

    Parameters:
    -----------

    mus : array
        Onsite energy inside the quantum Hall region.
    Bs : array
        Magnetic fields.
    params : dict
        System parameters.
    lview : Load-balanced view.
    """
    inps = product(mus, Bs)
    pars = np.array(list(inps))

    def calculate_GNS(xy):
        mu, B = xy
        _params = {**params, **{"mu": mu, "B": B}}
        from modules import utils

        return utils.transport(device=device, params=_params, quantity="G_NS")

    data = lview.map_async(calculate_GNS, pars)
    data.wait_interactive()
    data = data.get()
    data = np.reshape(data, (len(mus), len(Bs)))

    if params["spinful"]:
        degeneracy = 1
    else:
        degeneracy = 2

    dataset = xr.Dataset(
        data_vars={"GNS": (("mus", "Bs"), degeneracy * data)},
        coords={"mus": mus, "Bs": Bs},
    )

    attributes = {
        k: v
        for k, v in params.items()
        if v is not None and v is not False and v is not True
    }

    dataset.attrs = attributes

    return dataset


def plot_and_save(dataset):
    """
    Plot and save data.

    Read file:
    read_data = np.load('data_file.npy',allow_pickle='TRUE').item()
    use `read_dictionary` the same way as the `data`

    Parameters:
    -----------

    dataset : xarray dataset
    name : string
        File name.
    """
    print(dataset.attrs["name"])
    dataset.to_netcdf("./data/GNS_mu_B_" + dataset.attrs["name"] + ".nc", dataset)
    plot_GNS(dataset, save=True)


def plot_bands(dataset, ax=None, show_y_label=True, show_clb=False):
    """
    Plot bands.

    ax : matplotlib axis
    mu : float
        Chemical potential at the QH region.
    mu_SC : float
        Chemical potential at the SC region.
    B : float
        Magnetic field.
    interface : string
        NS interface orientation 'armchair' or 'zizgzag'.
    square : bool
        Square superconductor.
    spinful : bool
        Spinful calculation.
    show_y_label : bool
    """
    if ax:
        ax = ax
    else:
        fig, ax = plt.subplots()

    xr.plot.scatter(
        dataset,
        "kpoints",
        "energies",
        hue="xpect",
        cmap="Greys",
        s=1,
        alpha=0.1,
        add_guide=False,
        ax=ax,
        rasterized=True,
        vmin=0,
        vmax=0.1,
    )
    if show_y_label:
        ax.set_ylabel(r"$E(k) - E_F\ [t]$", labelpad=0)
    else:
        ax.set_ylabel(r"", labelpad=0)
    ax.set_xlabel(r"$k\ \left[\frac{1}{a}\right]$")
    ax.set_xlim(-np.pi, np.pi)
    ax.set_xticks([-np.pi, -np.pi / 2, 0, np.pi / 2, np.pi])
    ax.set_xticklabels([r"$-\pi$", r"$-\pi/2$", r"$0$", r"$\pi/2$", r"$\pi$"])
    ax.set_ylim(-0.2, 0.2)
    ax.axhline(y=0, c="k", ls="--")


def proj_bands(k, params, wrap_sys, density):
    new_params_ = dict(params, **dict(k_x=k))
    H_k = wrap_sys.hamiltonian_submatrix(params=new_params_, sparse=False)
    evals, evecs = np.linalg.eigh(H_k)
    xpect_k = []
    for i in range(len(evecs)):
        xpect_k.append(density(evecs[:, i]))
    np.asarray(xpect_k)

    return evals, xpect_k
