# # Graphene Quantum Hall/Superconductor: transport calculations
# ---

# ## Set cluster
# ---
# Connect with `hpc05`.

# +
# Set hpc05 engines
use_cluster = True

if use_cluster:
    import hpc05
    client, dview, lview = hpc05.start_remote_and_connect(
        200,
        folder="~/Work/graphene-sc-qhe/main_code/Numerics/"
    )

if not use_cluster:
    import ipyparallel
    rc = ipyparallel.Client()
    lview = rc.load_balanced_view()
    dview = rc[:]

# -

# ## Imports
# ---

# +
# %%px --local

# Cluster imports
from functools import partial
from modules import system, utils
from itertools import product
import numpy as np
from os import chdir

# -

# ## Compute and plot $G_{xx}$

# ## Create system

# ### Default values

# +
# %%px --local

# List with values of magnetic field
Bs = np.linspace(1, 6, num=100)

# List with values of chemical potential
mus = np.linspace(1e-3, 0.3, num=100)

## System dimensions (all in [nm])
dimensions = dict(
    L_contact=500,  ## Lenght of SC region
    L_device=500,  ## Scattering region lenght
    W_device=606,  ## Width of SC region and right lead
    W_lead=250,  ## Width of upper and lower leads
    lB_min=30,  ## Minimum value of magnetic length (for 1T)
)

## Default parameters
default_params = dict(
    Delta=1.3e-3 / 0.28,  ## Superconducting gap [t]
    Delta_lead=1.3e-3 / 0.28,  ## Superconducting gap at the lead [t]
    mu_SC=None,  ## Chemical potential at the superconducting region ('None' means that it is the same of normal region)
    delta_interface=None,  ## Zero for no interface disorder
    delta_boundary=None,  ## Zero for no boundary disorder
    l=50,  ## Smoothing lenght
    spinful=False,
)


# -

# %%px --local
def create_system(
    interface="zigzag", spinful=False, SC_square_bool=True, scaling_factor=10
):

    # Create system

    if interface == "zigzag":
        dimensions["theta"] = np.pi / 6
    elif interface == "armchair":
        dimensions["theta"] = 0

    ## Initialize device
    device = system.Device(
        dimensions=dimensions,
        scaling_factor=scaling_factor,
        interface=interface,
        spinful=spinful,
    )

    # Create device
    device.make_system(SC_square_bool=SC_square_bool)

    return device


# ## Compute and save

systems = {
    "spinless": {
        "honeycomb_sc": {
            "low_fermi_mistmatch": {
                "params": dict(mu_SC=0.2),
                "interface": "zigzag",
                "spin": False,
                "sc_bool": False,
                "scaling": 10,
            },
            "high_fermi_mistmatch": {
                "params": dict(mu_SC=0.5),
                "interface": "zigzag",
                "spin": False,
                "sc_bool": False,
                "scaling": 10,
            },
            "tunneling": {
                "params": dict(mu_SC=0.5, Delta_lead=0.0),
                "interface": "zigzag",
                "spin": False,
                "sc_bool": False,
                "scaling": 20,
            },
            "interface_disorder": {
                "params": dict(delta_interface=1.0),
                "interface": "zigzag",
                "spin": False,
                "sc_bool": False,
                "scaling": 10,
            },
            "boundary_disorder": {
                "params": dict(delta_boundary=1.0),
                "interface": "zigzag",
                "spin": False,
                "sc_bool": False,
                "scaling": 10,
            },
        },
        "square_sc": {
            "square_armchair": {
                "params": dict(),
                "interface": "armchair",
                "spin": False,
                "sc_bool": True,
                "scaling": 10,
            },
            "square_zigzag": {
                "params": dict(),
                "interface": "zigzag",
                "spin": False,
                "sc_bool": True,
                "scaling": 10,
            },
        },
    },
}

# +
chdir("files/transport")
for spin in systems:
    chdir(spin)
    systs_spin = systems[spin]
    for sc_lattice in systs_spin:
        chdir(sc_lattice)
        setups = systs_spin[sc_lattice]
        for setup in setups:
            print("Calculating setup: " + setup)
            setup_pars = setups[setup]
            device = create_system(
                interface=setup_pars["interface"],
                spinful=setup_pars["spin"],
                SC_square_bool=setup_pars["sc_bool"],
                scaling_factor=setup_pars["scaling"],
            )

            # Calculate
            dataset = utils.GNS_mu_B(
                mus=mus,
                Bs=Bs,
                params={
                    **default_params,
                    **dimensions,
                    **setup_pars["params"],
                    **dict(spinful=setup_pars["spin"], name=setup),
                },
                device=device,
                lview=lview,
            )

            # Plot and save
            utils.plot_and_save(dataset)
        chdir("..")
    chdir("..")
chdir("..")

if use_cluster:
    hpc05.kill_remote_ipcluster()
